"use strict";
// Copyright (c) Microsoft Corporation. All rights reserved. Licensed under the MIT license.
// See LICENSE in the project root for license information.
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadTypeScriptToolAsync = loadTypeScriptToolAsync;
const semver_1 = __importDefault(require("semver"));
const node_core_library_1 = require("@rushstack/node-core-library");
const OLDEST_SUPPORTED_TS_MAJOR_VERSION = 2;
const OLDEST_SUPPORTED_TS_MINOR_VERSION = 9;
const NEWEST_SUPPORTED_TS_MAJOR_VERSION = 5;
const NEWEST_SUPPORTED_TS_MINOR_VERSION = 8;
/**
 * @internal
 */
async function loadTypeScriptToolAsync(options) {
    const { terminal, heftConfiguration, buildProjectReferences, onlyResolveSymlinksInNodeModules } = options;
    const typeScriptToolPath = await heftConfiguration.rigPackageResolver.resolvePackageAsync('typescript', terminal);
    // Determine the compiler version
    const compilerPackageJsonFilename = `${typeScriptToolPath}/package.json`;
    const packageJson = await node_core_library_1.JsonFile.loadAsync(compilerPackageJsonFilename);
    const typescriptVersion = packageJson.version;
    const typescriptParsedVersion = semver_1.default.parse(typescriptVersion);
    if (!typescriptParsedVersion) {
        throw new Error(`Unable to parse version "${typescriptVersion}" for TypeScript compiler package in: ` +
            compilerPackageJsonFilename);
    }
    // Detect what features this compiler supports.  Note that manually comparing major/minor numbers
    // loosens the matching to accept prereleases such as "3.6.0-dev.20190530"
    const capabilities = {
        incrementalProgram: false,
        solutionBuilder: typescriptParsedVersion.major >= 3
    };
    if (typescriptParsedVersion.major > 3 ||
        (typescriptParsedVersion.major === 3 && typescriptParsedVersion.minor >= 6)) {
        capabilities.incrementalProgram = true;
    }
    if (buildProjectReferences && !capabilities.solutionBuilder) {
        throw new Error(`Building project references requires TypeScript@>=3.0, but the current version is ${typescriptVersion}`);
    }
    // Report a warning if the TypeScript version is too old/new.  The current oldest supported version is
    // TypeScript 2.9. Prior to that the "ts.getConfigFileParsingDiagnostics()" API is missing; more fixups
    // would be required to deal with that.  We won't do that work unless someone requests it.
    if (typescriptParsedVersion.major < OLDEST_SUPPORTED_TS_MAJOR_VERSION ||
        (typescriptParsedVersion.major === OLDEST_SUPPORTED_TS_MAJOR_VERSION &&
            typescriptParsedVersion.minor < OLDEST_SUPPORTED_TS_MINOR_VERSION)) {
        // We don't use writeWarningLine() here because, if the person wants to take their chances with
        // a seemingly unsupported compiler, their build should be allowed to succeed.
        terminal.writeLine(`The TypeScript compiler version ${typescriptVersion} is very old` +
            ` and has not been tested with Heft; it may not work correctly.`);
    }
    else if (typescriptParsedVersion.major > NEWEST_SUPPORTED_TS_MAJOR_VERSION ||
        (typescriptParsedVersion.major === NEWEST_SUPPORTED_TS_MAJOR_VERSION &&
            typescriptParsedVersion.minor > NEWEST_SUPPORTED_TS_MINOR_VERSION)) {
        terminal.writeLine(`The TypeScript compiler version ${typescriptVersion} is newer` +
            ' than the latest version that was tested with Heft ' +
            `(${NEWEST_SUPPORTED_TS_MAJOR_VERSION}.${NEWEST_SUPPORTED_TS_MINOR_VERSION}); it may not work correctly.`);
    }
    const ts = require(typeScriptToolPath);
    let realpath = ts.sys.realpath;
    if (onlyResolveSymlinksInNodeModules) {
        const resolver = new node_core_library_1.RealNodeModulePathResolver();
        realpath = resolver.realNodeModulePath;
    }
    return {
        tool: {
            ts,
            system: {
                ...ts.sys,
                realpath
            },
            typeScriptToolPath
        },
        typescriptVersion,
        typescriptParsedVersion,
        capabilities
    };
}
//# sourceMappingURL=loadTypeScriptTool.js.map